
/*
 Port Scan detection.

 */

#include <pcap.h>

#include "amesis.c"

#define HASH_SIZE	1 << 12
#define LIST_SIZE	0x1000

#define HASH(x) ((((x) >> HASH_SIZE) ^ (x % (1 << HASH_SIZE))) % LIST_SIZE)

typedef struct _ahost {
        struct _ahost  *next;
        struct _ahost  *prev;
        struct in_addr  addr;
        time_t          timestamp;
        unsigned short  counter;
        unsigned short  fwrule_div;
        unsigned short  fwrule_rej;
}               ahost;

typedef struct _alist {
        ahost           hastlist[LIST_SIZE];
        ahost          *addrlist[1 << HASH_SIZE];       /* ptr vers hashlist */
}               alist;


struct _ahost  *
lookup_ahost(struct in_addr h)
{
        ahost          *t;

        if ((t = hosts.addrlist[HASH(h.s_addr)]) == NULL)
                return (NULL);

        while (t && t->addr.s_addr != h.s_addr) {
                t = t->next;
        }
        return (t);
}


struct _ahost  *
add_ahost(struct in_addr h)
{
        /*
         * This function doesn't look for the existence in the list of the
         * addr. So you should take care about using it well.
         */

        /* Firstly, looks up for a empty case */

        int             i;
        struct _ahost  *new, *tmp;

        for (i = 0; i < LIST_SIZE; i++) {
                if (hosts.hastlist[i].addr.s_addr == 0)
                        break;
        }

        /* XXX No empty box ;-(( */

        if (i == LIST_SIZE) {
                log_debug("Hash list: no free box for %s\n", inet_ntoa(h));
                return (NULL);
        }
        log_debug("Hash list: adding host %s\n", inet_ntoa(h));

        new = &hosts.hastlist[i];
        new->addr = h;
        new->counter = 0;
        tmp = hosts.addrlist[HASH(h.s_addr)];

        new->prev = NULL;

        if (tmp) {
                new->next = tmp;
                new->next->prev = new;
        } else
                new->next = NULL;

        hosts.addrlist[HASH(h.s_addr)] = new;

        return (new);
}


struct _ahost  *
inc_counter(struct in_addr h)
{
        struct _ahost  *r;

        if ((r = lookup_ahost(h)) == NULL)
                r = add_ahost(h);

        r->counter++;
        r->timestamp = time(NULL);

        return (r);
}

int
rem_ahost(struct in_addr h)
{
        ahost          *t, *ot = NULL;

        if ((t = hosts.addrlist[HASH(h.s_addr)]) == NULL)
                return (0);


        while (t && t->addr.s_addr != h.s_addr) {
                ot = t;
                t = t->next;
        }

        if (t) {
                if (ot) {
                        ot->next = t->next;
                        t->next->prev = ot;
                } else {
                        hosts.addrlist[HASH(h.s_addr)] = t->next;
                        t->next->prev = NULL;
                }

                bzero(t, sizeof(ahost));
        }
        return (0);
}



/* Port scanners detections */
int
scan_detect(pifs tab_if, const unsigned char *pack, struct pcap_pkthdr h)
{


	return 0;
}
